@@ -100,7 +100,7 @@ struct Mtz {
     const Dataset& dataset() const { return parent->dataset(dataset_id); }
     bool has_data() const { return parent->has_data(); }
     int size() const { return has_data() ? parent->nreflections : 0; }
-    unsigned stride() const { return (unsigned) parent->columns.size(); }
+    size_t stride() const { return parent->columns.size(); }
     float& operator[](int n) { return parent->data[idx + n * stride()]; }
     float operator[](int n) const { return parent->data[idx + n * stride()]; }
     float& at(int n) { return parent->data.at(idx + n * stride()); }
@@ -139,6 +139,10 @@ struct Mtz {
     std::vector<float> floats;
     std::vector<std::string> axes;
 
+    UnitCell get_cell() const {
+      return UnitCell(floats[0], floats[1], floats[2],
+                      floats[3], floats[4], floats[5]);
+    }
     void set_cell(const UnitCell& uc) {
       floats[0] = (float) uc.a;
       floats[1] = (float) uc.b;
@@ -149,6 +153,13 @@ struct Mtz {
     }
 
     int dataset_id() const { return ints[20]; }
+    float phi_start() const { return floats[36]; }
+    float phi_end() const { return floats[37]; }
+    Mat33 matrix_U() const {
+      return Mat33(floats[6], floats[9],  floats[12],
+                   floats[7], floats[10], floats[13],
+                   floats[8], floats[11], floats[14]);
+    }
   };
 
   bool same_byte_order = true;
@@ -637,6 +648,14 @@ struct Mtz {
     return indices;
   }
 
+  Miller get_hkl(size_t offset) const {
+    return {{(int)data[offset], (int)data[offset+1], (int)data[offset+2]}};
+  }
+  void set_hkl(size_t offset, const Miller& hkl) {
+    for (int i = 0; i != 3; ++i)
+      data[offset + i] = static_cast<float>(hkl[i]);
+  }
+
   // (for unmerged MTZ only) change HKL according to M/ISYM
   bool switch_to_original_hkl() {
     if (!has_data())
@@ -651,8 +670,7 @@ struct Mtz {
     for (size_t n = 0; n + col->idx < data.size(); n += columns.size()) {
       int isym = static_cast<int>(data[n + col->idx]) & 0xFF;
       const Op& op = inv_symops.at((isym - 1) / 2);
-      std::array<int,3> hkl = {{(int)data[n], (int)data[n+1], (int)data[n+2]}};
-      hkl = op.apply_to_hkl(hkl);
+      Miller hkl = op.apply_to_hkl(get_hkl(n));
       int sign = (isym & 1) ? 1 : -1;
       for (int i = 0; i < 3; ++i)
         data[n+i] = static_cast<float>(sign * hkl[i]);
@@ -670,10 +688,9 @@ struct Mtz {
     int misym_idx = col->idx;
     UnmergedHklMover hkl_mover(spacegroup);
     for (size_t n = 0; n + col->idx < data.size(); n += columns.size()) {
-      std::array<int,3> hkl{{(int)data[n], (int)data[n+1], (int)data[n+2]}};
-      int isym = hkl_mover.move_to_asu(hkl);
-      for (int i = 0; i != 3; ++i)
-        data[n + i] = (float)hkl[i];
+      Miller hkl = get_hkl(n);
+      int isym = hkl_mover.move_to_asu(hkl);  // modifies hkl
+      set_hkl(n, hkl);
       float& misym = data[n + misym_idx];
       misym = float(((int)misym & ~0xff) | isym);
     }
@@ -765,7 +782,7 @@ struct MtzDataProxy {
   const UnitCell& unit_cell() const { return mtz_.cell; }
   const SpaceGroup* spacegroup() const { return mtz_.spacegroup; }
   Miller get_hkl(size_t offset, const std::array<size_t,3>&) const {
-    return {{get_int(offset + 0), get_int(offset + 1), get_int(offset + 2)}};
+    return mtz_.get_hkl(offset);
   }
 };
 
